function[bp, cbp]=create_DOPs(s1,s2,Ecut)
% [bp cbp]=create_DOPs(s1,s2,Ecut)
% creates scale derivatives of a DOP quadrature filter
% BP and CBP (complex), based on the parameters
% S1: finer scale of DOP (at least: 1)
% S2: coarser scale of DOP (must be larger than S1)
% Ecut: maximal cut-off energy (in percentage)
 
% Copyright (c) 2003 Michael Felsberg
% $Revision: 1.0 $  $Date: 2003/09/02 12:55:12 $
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program; if not, write to the Free Software
%    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

if nargin<3
  error('Scales and cut-off energy must be specified!');
end
E=(1-Ecut/100)*3/16/pi/s2^4;
S=s2;
[x,y]=meshgrid(-S:S,-S:S);
tp2=(-2*s2^2+x.^2+y.^2)./(2*pi)./(s2^2+x.^2+y.^2).^(5/2);
while sum(tp2(:).^2)<E
  S=S+1;
  [x,y]=meshgrid(-S:S,-S:S);
  tp2=(-2*s2^2+x.^2+y.^2)./(2*pi)./(s2^2+x.^2+y.^2).^(5/2);
end
tp1=(-2*s1^2+x.^2+y.^2)./(2*pi)./(s1^2+x.^2+y.^2).^(5/2);
ntp1=sum(tp1(:));
ntp2=sum(tp2(:));
bp=tp1./ntp1-tp2./ntp2; % ensure zero DC
c11=-3*s1*x./(2*pi*(s1^2+x.^2+y.^2).^(5/2));
c12=-3*s2*x./(2*pi*(s2^2+x.^2+y.^2).^(5/2));
c21=-3*s1*y./(2*pi*(s1^2+x.^2+y.^2).^(5/2));
c22=-3*s2*y./(2*pi*(s2^2+x.^2+y.^2).^(5/2));
c1bp=c11./ntp1-c12./ntp2;
c2bp=c21./ntp1-c22./ntp2;
cbp=c1bp+i*c2bp;
bp=sqrt(sum(real(cbp(:).*conj(cbp(:))))/sum(bp(:).^2))*bp; % ensure same energy