function[P,Q]=gGETs(bild,sigma)
% function[T,phi,msk]=gGETs(bild,sigma)
%
% Computes the gradient energy tensor for an image <bild>, using Gaussian
% derivatives with variance <sigma>^2, 2<sigma>^2, and 3<sigma>^2.
% <T> contains the tensor components in a 3D array T11,T12,T22.
% <phi> contains the local phase for i1D signals.
% <msk> is true for admissible tensors (positive definite) and false
% otherwise.

% Copyright 2004 Michael Felsberg
% Version 2.0

% creating filters
sigma=sqrt(2)*sigma;
s=sigma;
vs=ceil(4*s);
[x,y]=meshgrid(-vs:vs,-vs:vs);
% rotation by pi/8 gives better isotropy
ro=pi/8;
xd=cos(ro)*x+sin(ro)*y;
y=cos(ro)*y-sin(ro)*x;
x=xd;
r2=x.^2+y.^2;
% create Gaussian filters
g=1/2/pi/s^2*exp(-(r2)/2/s^2);
g=g./sum(g(:));
g=(r2-2*s^2)./s^4.*g;
% create derivatives
gx=-x.*(r2-4*s^2)./(r2-2*s^2+eps).*g./s^2;
gy=-y.*(r2-4*s^2)./(r2-2*s^2+eps).*g./s^2;

s=sqrt(2)*sigma;
g=1/2/pi/s^2*exp(-(r2)/2/s^2);
g=g./sum(g(:));
g=(r2-2*s^2)./s^4.*g;
gxx=(x.^2.*r2+4*s^4-s^2*(7*x.^2+y.^2))./(r2-2*s^2).*g./s^4;
gxy=x.*y.*(r2-6*s^2)./(r2-2*s^2).*g./s^4;
gyy=(y.^2.*r2+4*s^4-s^2*(7*y.^2+x.^2))./(r2-2*s^2).*g./s^4;

s=sqrt(3)*sigma;
g=1/2/pi/s^2*exp(-(r2)/2/s^2);
g=g./sum(g(:));
g=(r2-2*s^2)./s^4.*g;
Lx=-x.*(24*s^4-12*s^2*r2+r2.^2)./(r2-2*s^2).*g./s^6;
Ly=-y.*(24*s^4-12*s^2*r2+r2.^2)./(r2-2*s^2).*g./s^6;

% apply operators
cx=imfilter(bild,gx);
cy=imfilter(bild,gy);
cxx=imfilter(bild,gxx);
cyy=imfilter(bild,gyy);
cxy=imfilter(bild,gxy);
cLx=imfilter(bild,Lx);
cLy=imfilter(bild,Ly);

% derotate
cd=exp(i*ro)*(cx+i*cy);
cx=real(cd);
cy=imag(cd);
cd=1/2*exp(i*2*ro)*(cxx-cyy+i*2*cxy);
cd2=1/2*(cxx+cyy);
cxx=real(cd)+cd2;
cyy=cd2-real(cd);
cxy=imag(cd);
cd=exp(i*ro)*(cLx+i*cLy);
cLx=real(cd);
cLy=imag(cd);

% generate tensors (odd and even)
Te=cxx.^2+cxy.^2;
Te(:,:,2)=(cxx+cyy).*cxy;
Te(:,:,3)=cyy.^2+cxy.^2;
To=-cx.*cLx;
To(:,:,2)=-1/2*(cx.*cLy+cy.*cLx);
To(:,:,3)=-cy.*cLy;
tro=To(:,:,1)+To(:,:,3);
tre=Te(:,:,1)+Te(:,:,3);
% mask for admissible tensors
%tol=1;
%msk=((tro>-tol)&(To(:,:,1).*To(:,:,3)-To(:,:,2).^2>-tol*tro.^2));
%msk=msk&((tre>-tol)&(Te(:,:,1).*Te(:,:,3)-Te(:,:,2).^2>-tol*tre.^2));
T=To+Te;
% phase:
o=T(:,:,1)-T(:,:,3)+i*2*T(:,:,2);
o=sqrt(o./(abs(o)+eps));
so=sign(real(o).*cx+imag(o).*cy);
o2=cxx.*real(o)+cxy.*imag(o)+i*(cxy.*real(o)+cyy.*imag(o));
o2=(o2)./(abs(o2)+eps);
st=sign(real(o).*real(o2)+imag(o).*imag(o2));
%phi=angle(-st.*tre-i*so.*tro);
P=st.*tre;
Q=-so.*tro.*o;